-----------------------------------------------------------------------------------------------
-- Client Lua Script for CustomTelegraphs
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- CustomTelegraphs Module Definition
-----------------------------------------------------------------------------------------------
local CustomTelegraphs = {
	name = "Custom Telegraphs",
	version = {0,0,1},
	settings = {
		bEnabled = true,
		bPvP = false,
		currentColorName= "white"
	},
	tColors = 
		{		
		["white"]={ 255, 255, 255, 100}, -- white
		["red"]={255,   0,   0, 100}, -- red
		["yellow"]={255, 255,   0, 100}, -- yellow
		["cyan"]={0, 255, 255, 100}, -- cyan
		["purple"]={255,   0, 255, 100}, -- purple
		["blue"]={0, 0, 255, 100} -- blue
		}
		
} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CustomTelegraphs:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

--     initialize variables here

    return o
end

function CustomTelegraphs:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	self:SaveDefaults()
	self:OnCharacterCreated()

end
 
function CustomTelegraphs:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end	
	return {version = self.version, settings = self.settings}
end

function CustomTelegraphs:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if not tData.version or tData.version[2] < 3 then
		tData.settings = nil
	elseif tData.version[2] < 4 then
		tData.settings.bPvP = false
	end

	self.settings = tData.settings or self.settings
end


function CustomTelegraphs:OnChangeWorld()
	-- If the player changes zone while in combat, no out of combat event will be fired,
	-- so we have to deactivate manually in that case


	if self.settings.bPvP and MatchingGame.IsInPVPGame() then
		Apollo.RemoveEventHandler("UnitEnteredCombat", self)
		

	else
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnCombat", self)
	end
end


function CustomTelegraphs:SaveDefaults()
	self.nOriginalColorSet = Apollo.GetConsoleVariable("spell.telegraphColorSet")
	self.tOriginalColor = {
		Apollo.GetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorR"),
		Apollo.GetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorG"),
		Apollo.GetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorB"),
		Apollo.GetConsoleVariable("spell.fillOpacityCustom1_34"),
	}
	if self.settings.bPvP then
		self.tOriginalPvPColor = {
			Apollo.GetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorR"),
			Apollo.GetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorG"),
			Apollo.GetConsoleVariable("spell.custom1EnemyPlayerDetrimentalTelegraphColorB"),
		}
	end
end

function CustomTelegraphs:RestoreDefaults()
	self:SetColor(self.tOriginalColor)
	if self.settings.bPvP then self:SetPvPColor(self.tOriginalPvPColor) end
	Apollo.SetConsoleVariable("spell.telegraphColorSet", self.nOriginalColorSet)
end


function CustomTelegraphs:SetColor(sColorName)
		self.settings.currentColorName=sColorName
		local tColor =  self.tColors[sColorName]
	Apollo.SetConsoleVariable("spell.telegraphColorSet", 4)

	Apollo.SetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorR", tColor[1])
	Apollo.SetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorG", tColor[2])
	Apollo.SetConsoleVariable("spell.custom1EnemyNPCDetrimentalTelegraphColorB", tColor[3])
	Apollo.SetConsoleVariable("spell.fillOpacityCustom1_34", tColor[4])
	GameLib.RefreshCustomTelegraphColors()

	
end

-----------------------------------------------------------------------------------------------
-- CustomTelegraphs OnLoad
-----------------------------------------------------------------------------------------------
function CustomTelegraphs:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CustomTelegraphs.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	self.tmrInit = ApolloTimer.Create(0, false, 'Init', self) -- Wait 0s to let OnRestore fire
end

-----------------------------------------------------------------------------------------------
-- CustomTelegraphs OnDocLoaded
-----------------------------------------------------------------------------------------------
function CustomTelegraphs:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CustomTelegraphsForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("ct", "OnSlashCommand", self)


		-- Do additional Addon initialization here
	end
end

CustomTelegraphs.SlashCommands = setmetatable({

	setColor   = function(sColorName)
			CustomTelegraphs:SetColor(sColorName)

		end	

},{
	__index = function(t, key) return function()
		Print(CustomTelegraphs:GetName() .. " - Available commands: ")
		for k,v in pairs(t) do Print("  /ct "..k) end
	end end
	
})


function CustomTelegraphs:OnSlashCommand(strCmd, strParams)
	if strParams == "" then	self.SlashCommands.toggle()
	else
		local tArgs = {}
		for arg in strParams:gmatch("[^%s]+") do
			table.insert(tArgs, arg)
		end

		CustomTelegraphs.SlashCommands[tArgs[1]](unpack(tArgs, 2, #tArgs))
	end
end



function CustomTelegraphs.GetName()
	return "Custom Telegraphs by Seal Slicer"
end


-----------------------------------------------------------------------------------------------
-- CustomTelegraphs Functions
-----------------------------------------------------------------------------------------------
function CustomTelegraphs:OnCharacterCreated()
	local unit = GameLib.GetPlayerUnit()
	if not unit then Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self); return end
	Apollo.RemoveEventHandler("CharacterCreated")
	self:Refresh()
end

function CustomTelegraphs:Refresh()
	Apollo.RemoveEventHandler("UnitEnteredCombat", self)
	Apollo.RemoveEventHandler("ChangeWorld", self)
	
	local unitPlayer = GameLib.GetPlayerUnit()
	if self.settings.bEnabled then
		Apollo.RegisterEventHandler("ChangeWorld", "OnChangeWorld", self)		
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnCombat", self)
		
	end
end

-----------------------------------------------------------------------------------------------
-- CustomTelegraphs Instance
-----------------------------------------------------------------------------------------------
local CustomTelegraphsInst = CustomTelegraphs:new()
CustomTelegraphsInst:Init()

